@@ -4,17 +4,13 @@ import os
 import re
 import sys
 import types
+from collections import OrderedDict
 
 if sys.version_info >= (3, 0):
     from io import StringIO
 else:
     from StringIO import StringIO
 
-if sys.version_info < (2, 7):
-    from ordereddict import OrderedDict
-else:
-    from collections import OrderedDict
-
 
 ACTION_TYPES_THAT_DONT_NEED_A_VALUE = (argparse._StoreTrueAction,
     argparse._StoreFalseAction, argparse._CountAction,
@@ -152,8 +148,8 @@ class DefaultConfigFileParser(ConfigFileParser):
             if not line or line[0] in ["#", ";", "["] or line.startswith("---"):
                 continue
             white_space = "\\s*"
-            key = "(?P<key>[^:=;#\s]+?)"
-            value = white_space+"[:=\s]"+white_space+"(?P<value>.+?)"
+            key = r"(?P<key>[^:=;#\s]+?)"
+            value = white_space+r"[:=\s]"+white_space+"(?P<value>.+?)"
             comment = white_space+"(?P<comment>\\s[;#].*)?"
 
             key_only_match = re.match("^" + key + comment + "$", line)
@@ -174,7 +170,7 @@ class DefaultConfigFileParser(ConfigFileParser):
                 items[key] = value
                 continue
 
-            raise ConfigFileParserException("Unexpected line %s in %s: %s" % (i,
+            raise ConfigFileParserException("Unexpected line {} in {}: {}".format(i,
                 getattr(stream, 'name', 'stream'), line))
         return items
 
@@ -187,7 +183,7 @@ class DefaultConfigFileParser(ConfigFileParser):
             if isinstance(value, list):
                 # handle special case of lists
                 value = "["+", ".join(map(str, value))+"]"
-            r.write("%s = %s\n" % (key, value))
+            r.write("{} = {}\n".format(key, value))
         return r.getvalue()
 
 
@@ -265,22 +261,7 @@ class ArgumentParser(argparse.ArgumentParser):
     environment variables and .ini or .yaml-style config files.
     """
 
-    def __init__(self,
-        add_config_file_help=True,
-        add_env_var_help=True,
-        auto_env_var_prefix=None,
-        default_config_files=[],
-        ignore_unknown_config_file_keys=False,
-        config_file_parser_class=DefaultConfigFileParser,
-        args_for_setting_config_path=[],
-        config_arg_is_required=False,
-        config_arg_help_message="config file path",
-        args_for_writing_out_config_file=[],
-        write_out_config_file_arg_help_message="takes the current command line "
-            "args and writes them out to a config file at the given path, then "
-            "exits",
-        **kwargs
-        ):
+    def __init__(self, *args, **kwargs):
 
         """Supports args of the argparse.ArgumentParser constructor
         as **kwargs, as well as the following additional args.
@@ -331,11 +312,34 @@ class ArgumentParser(argparse.ArgumentParser):
             write_out_config_file_arg_help_message: The help message to use for
                 the args in args_for_writing_out_config_file.
         """
+        # This is the only way to make positional args (tested in the argparse
+        # main test suite) and keyword arguments work across both Python 2 and
+        # 3. This could be refactored to not need extra local variables.
+        add_config_file_help = kwargs.pop('add_config_file_help', True)
+        add_env_var_help = kwargs.pop('add_env_var_help', True)
+        auto_env_var_prefix = kwargs.pop('auto_env_var_prefix', None)
+        default_config_files = kwargs.pop('default_config_files', [])
+        ignore_unknown_config_file_keys = kwargs.pop(
+            'ignore_unknown_config_file_keys', False)
+        config_file_parser_class = kwargs.pop('config_file_parser_class',
+                                              DefaultConfigFileParser)
+        args_for_setting_config_path = kwargs.pop(
+            'args_for_setting_config_path', [])
+        config_arg_is_required = kwargs.pop('config_arg_is_required', False)
+        config_arg_help_message = kwargs.pop('config_arg_help_message',
+                                             "config file path")
+        args_for_writing_out_config_file = kwargs.pop(
+            'args_for_writing_out_config_file', [])
+        write_out_config_file_arg_help_message = kwargs.pop(
+            'write_out_config_file_arg_help_message', "takes the current "
+            "command line args and writes them out to a config file at the "
+            "given path, then exits")
+
         self._add_config_file_help = add_config_file_help
         self._add_env_var_help = add_env_var_help
         self._auto_env_var_prefix = auto_env_var_prefix
 
-        argparse.ArgumentParser.__init__(self, **kwargs)
+        argparse.ArgumentParser.__init__(self, *args, **kwargs)
 
         # parse the additional args
         if config_file_parser_class is None:
@@ -394,7 +398,7 @@ class ArgumentParser(argparse.ArgumentParser):
             args = list(args)
 
         # normalize args by converting args like --key=value to --key value
-        normalized_args = list()
+        normalized_args = []
         for arg in args:
             if arg and arg[0] in self.prefix_chars and '=' in arg:
                 key, value =  arg.split('=', 1)
@@ -431,6 +435,7 @@ class ArgumentParser(argparse.ArgumentParser):
 
         # add env var settings to the commandline that aren't there already
         env_var_args = []
+        nargs = False
         actions_with_env_var_values = [a for a in self._actions
             if not a.is_positional_arg and a.env_var and a.env_var in env_vars
                 and not already_on_command_line(args, a.option_strings)]
@@ -439,20 +444,23 @@ class ArgumentParser(argparse.ArgumentParser):
             value = env_vars[key]
             # Make list-string into list.
             if action.nargs or isinstance(action, argparse._AppendAction):
+                nargs = True
                 element_capture = re.match('\[(.*)\]', value)
                 if element_capture:
                     value = [val.strip() for val in element_capture.group(1).split(',') if val.strip()]
             env_var_args += self.convert_item_to_command_line_arg(
                 action, key, value)
 
-        args = args + env_var_args
+        if nargs:
+            args = args + env_var_args
+        else:
+            args = env_var_args + args
 
         if env_var_args:
             self._source_to_settings[_ENV_VAR_SOURCE_KEY] = OrderedDict(
                 [(a.env_var, (a, env_vars[a.env_var]))
                     for a in actions_with_env_var_values])
 
-
         # before parsing any config files, check if -h was specified.
         supports_help_arg = any(
             a for a in self._actions if isinstance(a, argparse._HelpAction))
@@ -460,8 +468,8 @@ class ArgumentParser(argparse.ArgumentParser):
             "-h" in args or "--help" in args)
 
         # prepare for reading config file(s)
-        known_config_keys = dict((config_key, action) for action in self._actions
-            for config_key in self.get_possible_config_keys(action))
+        known_config_keys = {config_key: action for action in self._actions
+            for config_key in self.get_possible_config_keys(action)}
 
         # open the config file(s)
         config_streams = []
@@ -484,6 +492,7 @@ class ArgumentParser(argparse.ArgumentParser):
 
             # add each config item to the commandline unless it's there already
             config_args = []
+            nargs = False
             for key, value in config_items.items():
                 if key in known_config_keys:
                     action = known_config_keys[key]
@@ -503,9 +512,14 @@ class ArgumentParser(argparse.ArgumentParser):
                     if source_key not in self._source_to_settings:
                         self._source_to_settings[source_key] = OrderedDict()
                     self._source_to_settings[source_key][key] = (action, value)
+                    if (action and action.nargs or
+                        isinstance(action, argparse._AppendAction)):
+                        nargs = True
 
-            args = args + config_args
-
+            if nargs:
+                args = args + config_args
+            else:
+                args = config_args + args
 
         # save default settings for use by print_values()
         default_settings = OrderedDict()
@@ -553,7 +567,7 @@ class ArgumentParser(argparse.ArgumentParser):
                 with open(output_file_path, "w") as output_file:
                     pass
             except IOError as e:
-                raise ValueError("Couldn't open %s for writing: %s" % (
+                raise ValueError("Couldn't open {} for writing: {}".format(
                     output_file_path, e))
         if output_file_paths:
             # generate the config file contents
@@ -675,7 +689,7 @@ class ArgumentParser(argparse.ArgumentParser):
             args.append( command_line_key )
             args.append( value )
         else:
-            raise ValueError("Unexpected value type %s for value: %s" % (
+            raise ValueError("Unexpected value type {} for value: {}".format(
                 type(value), value))
 
         return args
@@ -692,7 +706,7 @@ class ArgumentParser(argparse.ArgumentParser):
             return keys
 
         for arg in action.option_strings:
-            if any([arg.startswith(2*c) for c in self.prefix_chars]):
+            if any(arg.startswith(2*c) for c in self.prefix_chars):
                 keys += [arg[2:], arg] # eg. for '--bla' return ['bla', '--bla']
 
         return keys
@@ -771,7 +785,7 @@ class ArgumentParser(argparse.ArgumentParser):
             r.write(source)
             for key, (action, value) in settings.items():
                 if key:
-                    r.write("  %-19s%s\n" % (key+":", value))
+                    r.write("  {:<19}{}\n".format(key+":", value))
                 else:
                     if isinstance(value, str):
                         r.write("  %s\n" % value)
